@@ -186,8 +186,38 @@ static int sdla_te3_get_fe_status(sdla_fe_t *fe, unsigned char *status, int notu
  */
 static int sdla_te3_polling(sdla_fe_t *fe)
 {
-       	int err;
+    int err;
+	int prev_status=fe->fe_status;
+	sdla_t	*card = (sdla_t*)fe->card;
+
+
 	err=sdla_te3_read_alarms(fe, WAN_FE_ALARM_READ|WAN_FE_ALARM_UPDATE);
+	
+	DEBUG_TEST("%s: Polling prev_state = %i newsate=%i timeout=%i diff=%i\n",
+				fe->name,prev_status,fe->fe_status,fe->fe_cfg.cfg.te3_cfg.disc_cfg_timeout,
+				(SYSTEM_TICKS-fe->fe_param.te3.disc_timeout)/HZ);
+
+	if (fe->fe_cfg.cfg.te3_cfg.disc_cfg_timeout) {
+		if (prev_status == FE_CONNECTED) {
+			if (fe->fe_status == FE_DISCONNECTED) {
+				DEBUG_TEST("%s: FE_DISCONNECTED checking timeout %lu\n",
+					fe->name,	(SYSTEM_TICKS-fe->fe_param.te3.disc_timeout));
+	
+				if ((SYSTEM_TICKS-fe->fe_param.te3.disc_timeout) < (fe->fe_cfg.cfg.te3_cfg.disc_cfg_timeout*HZ)) {
+					DEBUG_EVENT("%s: FE Disconnect within disc timeout %i\n",
+								card->devname,fe->fe_cfg.cfg.te3_cfg.disc_cfg_timeout);
+					sdla_te3_config(fe);
+				}
+			}
+		} else {
+			if (fe->fe_status == FE_CONNECTED) {
+				fe->fe_param.te3.disc_timeout=SYSTEM_TICKS;
+				DEBUG_TEST("%s: FE_CONNECTED resetting timeout %lu\n",
+					fe->name,	(SYSTEM_TICKS-fe->fe_param.te3.disc_timeout)/HZ);
+			}
+		}
+	}
+
 	return err;
 }
 
@@ -903,6 +933,41 @@ static u32 sdla_te3_get_lb(sdla_fe_t *fe)
 	return type;
 }
 
+/******************************************************************************
+*				sdla_te3_udp_lb()	
+*
+* Description:
+* Arguments:	
+* Returns:
+******************************************************************************/
+static int sdla_te3_udp_lb(sdla_fe_t *fe, unsigned char *data)
+{
+	sdla_t			*card = (sdla_t*)fe->card;
+	sdla_fe_lbmode_t	*lb = (sdla_fe_lbmode_t*)data;
+	int			err = 0;
+
+	WAN_ASSERT(card == NULL);
+	if (lb->cmd == WAN_FE_LBMODE_CMD_SET){
+
+		/* Activate/Deactivate Line Loopback modes */
+		if (card->adptr_subtype == AFT_SUBTYPE_NORMAL){
+			err = sdla_te3_old_set_lb_modes(fe, lb->type, lb->mode); 
+		}else if (card->adptr_subtype == AFT_SUBTYPE_SHARK){
+			err = sdla_te3_set_lb_modes(fe, lb->type, lb->mode); 
+		}
+
+	}else if (lb->cmd == WAN_FE_LBMODE_CMD_SET){
+
+		if (card->adptr_subtype == AFT_SUBTYPE_NORMAL){
+			lb->type_map = sdla_te3_old_get_lb(fe);
+		}else if (card->adptr_subtype == AFT_SUBTYPE_SHARK){
+			lb->type_map= sdla_te3_get_lb(fe);
+		}
+	}else{
+		return -EINVAL;
+	}
+	return err;
+}
 
 /******************************************************************************
  *				sdla_te3_udp()	
@@ -914,11 +979,9 @@ static u32 sdla_te3_get_lb(sdla_fe_t *fe)
  */
 static int sdla_te3_udp(sdla_fe_t *fe, void *pudp_cmd, unsigned char *data)
 {
-	sdla_t			*card = (sdla_t*)fe->card;
 	wan_femedia_t		*fe_media = NULL;
 	sdla_fe_debug_t	*fe_debug = NULL;
 	wan_cmd_t		*udp_cmd = (wan_cmd_t*)pudp_cmd;
-	int			err = -EINVAL;
 
 	switch(udp_cmd->wan_cmd_command){
 	case WAN_GET_MEDIA_TYPE:
@@ -930,27 +993,13 @@ static int sdla_te3_udp(sdla_fe_t *fe, void *pudp_cmd, unsigned char *data)
 		break;
 
 	case WAN_FE_LB_MODE:
-		if (!data[0]){
-			u32	mode = 0;
-			if (card->adptr_subtype == AFT_SUBTYPE_NORMAL){
-				mode = sdla_te3_old_get_lb(fe);
-			}else if (card->adptr_subtype == AFT_SUBTYPE_SHARK){
-				mode = sdla_te3_get_lb(fe);
-			}
-		       	memcpy(&data[0], (u8*)&mode, sizeof(mode));
-		    	udp_cmd->wan_cmd_return_code = WAN_CMD_OK;
-    		    	udp_cmd->wan_cmd_data_len = sizeof(mode);
+		if (sdla_te3_udp_lb(fe, data)){
+		    	udp_cmd->wan_cmd_return_code	= WAN_UDP_FAILED_CMD;
+	    		udp_cmd->wan_cmd_data_len	= 0x00;
 			break;
 		}
-		/* Activate/Deactivate Line Loopback modes */
-		if (card->adptr_subtype == AFT_SUBTYPE_NORMAL){
-			err = sdla_te3_old_set_lb_modes(fe, data[0], data[1]); 
-		}else if (card->adptr_subtype == AFT_SUBTYPE_SHARK){
-			err = sdla_te3_set_lb_modes(fe, data[0], data[1]); 
-		}
-	    	udp_cmd->wan_cmd_return_code = 
-				(!err) ? WAN_CMD_OK : WAN_UDP_FAILED_CMD;
-	    	udp_cmd->wan_cmd_data_len = 0x00;
+	    	udp_cmd->wan_cmd_return_code = WAN_CMD_OK;
+	    	udp_cmd->wan_cmd_data_len = sizeof(sdla_fe_lbmode_t);
 		break;
 
 	case WAN_FE_GET_STAT:
@@ -1208,10 +1257,35 @@ static int sdla_te3_config(void *p_fe)
 	sdla_te3_cfg_t	*te3_cfg = &fe_cfg->cfg.te3_cfg;
 	u16		adptr_subtype;
 	unsigned char	data = 0x00;
+
+	DEBUG_EVENT("%s: TE3 Config\n",fe->name);
 	
 	WAN_ASSERT(fe->write_fe_reg == NULL);
 	WAN_ASSERT(fe->read_fe_reg == NULL);
 
+
+	if (fe_cfg->media == WAN_MEDIA_DS3) {
+		if (fe->fe_cfg.cfg.te3_cfg.disc_cfg_timeout == 0xFF) {
+			/* If user set disc_cfg_timeout to 0xFF then disable feature */
+			fe->fe_cfg.cfg.te3_cfg.disc_cfg_timeout=0;
+		} else if (fe->fe_cfg.cfg.te3_cfg.disc_cfg_timeout == 0) {
+			/* By default set timeout value to 10 sec */
+			fe->fe_cfg.cfg.te3_cfg.disc_cfg_timeout=10;
+		}
+
+		DEBUG_EVENT("%s: (T3) Disconnected timeout set to %i !\n",
+						fe->name, fe->fe_cfg.cfg.te3_cfg.disc_cfg_timeout);
+
+	} else {
+		if (fe->fe_cfg.cfg.te3_cfg.disc_cfg_timeout &&
+			fe->fe_cfg.cfg.te3_cfg.disc_cfg_timeout != 0xFF) {
+			DEBUG_EVENT("%s: (E3) Disconnected timeout not supported !\n",
+						fe->name);
+		}
+		fe->fe_cfg.cfg.te3_cfg.disc_cfg_timeout=0;
+	}	
+
+
 	card->hw_iface.getcfg(card->hw, SDLA_ADAPTERSUBTYPE, &adptr_subtype);
 	
 	data = READ_REG(0x02);
@@ -1232,11 +1306,13 @@ static int sdla_te3_config(void *p_fe)
 		DEBUG_TE3("%s: (T3/E3) Media type DS3\n",
 				fe->name);
 		data |= BIT_OPMODE_DS3;
+		
 		break;
 	case WAN_MEDIA_E3:
 		DEBUG_TE3("%s: (T3/E3) Media type E3\n",
 				fe->name);
 		data &= ~BIT_OPMODE_DS3;
+		fe->fe_cfg.cfg.te3_cfg.disc_cfg_timeout=0;
 		break;
 	default:
 		DEBUG_EVENT("%s: Invalid Media type 0x%X\n",
@@ -1329,8 +1405,7 @@ static int sdla_te3_config(void *p_fe)
 
 	DEBUG_EVENT("%s: %s disconnected!\n",
 					fe->name, FE_MEDIA_DECODE(fe));
-	sdla_te3_read_alarms(fe, 1);
-
+	//sdla_te3_read_alarms(fe, 1);
 	//sdla_te3_set_intr(fe);
 	return 0;
 }
@@ -1340,11 +1415,19 @@ static int sdla_te3_add_timer(sdla_fe_t* fe, unsigned long delay)
 {
 	int	err=0;
 
-	if (wan_test_bit(TE_TIMER_KILL,(void*)&fe->te_param.critical) ||
-	    wan_test_bit(TE_TIMER_RUNNING,(void*)&fe->te_param.critical)) {
-		return 0;
+	if (wan_test_bit(TE_TIMER_KILL,(void*)&fe->te3_param.critical)) {
+		DEBUG_EVENT("%s: Warning: sdla_te3_add_timer() timer killed!\n",fe->name);
+		return -EINVAL;
+
 	}
 
+	if (wan_test_bit(TE_TIMER_RUNNING,(void*)&fe->te3_param.critical)) {
+		DEBUG_EVENT("%s: Warning: sdla_te3_add_timer() timer already running!\n",fe->name);
+		return -EBUSY;
+	}
+	
+	wan_set_bit(TE_TIMER_RUNNING,(void*)&fe->te3_param.critical);
+
 #if defined(__WINDOWS__)
 	/* delay is in MS, so it can be used directly by wan_add_timer() */
 	err = wan_add_timer(&fe->timer, delay);
@@ -1352,11 +1435,13 @@ static int sdla_te3_add_timer(sdla_fe_t* fe, unsigned long delay)
 	err = wan_add_timer(&fe->timer, delay * HZ / 1000);
 #endif
 
-	if (err){
+	if (err) {
+		wan_clear_bit(TE_TIMER_RUNNING,(void*)&fe->te3_param.critical);
+		DEBUG_EVENT("%s: Error: Failed to add timer err=%i !\n",fe->name,err);
 		/* Failed to add timer */
 		return -EINVAL;
 	}
-	wan_set_bit(TE_TIMER_RUNNING,(void*)&fe->te_param.critical);
+
 	return 0;	
 }
 
@@ -1373,27 +1458,32 @@ static void sdla_te3_timer(unsigned long pfe)
 	sdla_fe_t	*fe = (sdla_fe_t*)pfe;
 	sdla_t 		*card = (sdla_t*)fe->card;
 	wan_device_t	*wandev = &card->wandev;
+	int err;
 
 	DEBUG_TEST("[TE3] %s: TE3 timer!\n", fe->name);
-	if (wan_test_bit(TE_TIMER_KILL,(void*)&fe->te_param.critical)){
-		wan_clear_bit(TE_TIMER_RUNNING,(void*)&fe->te_param.critical);
+
+	if (wan_test_bit(TE_TIMER_KILL,(void*)&fe->te3_param.critical)){
+		DEBUG_EVENT("WARNING: %s: %s() Timer has been killed!\n", 
+					fe->name,__FUNCTION__);
+		wan_clear_bit(TE_TIMER_RUNNING,(void*)&fe->te3_param.critical);
 		return;
 	}
-	if (!wan_test_bit(TE_TIMER_RUNNING,(void*)&fe->te_param.critical)){
+	if (!wan_test_bit(TE_TIMER_RUNNING,(void*)&fe->te3_param.critical)){
 		/* Somebody clear this bit */
-		DEBUG_EVENT("WARNING: %s: Timer bit is cleared (should never happened)!\n", 
-					fe->name);
+		DEBUG_EVENT("WARNING: %s: %s() Timer bit is cleared (should never happened)!\n", 
+					fe->name,__FUNCTION__);
 		return;
 	}
-	wan_clear_bit(TE_TIMER_RUNNING,(void*)&fe->te_param.critical);
+	wan_clear_bit(TE_TIMER_RUNNING,(void*)&fe->te3_param.critical);
 
 	if (wandev->fe_enable_timer){
 		wandev->fe_enable_timer(fe->card);
 	}
 
-	
-	sdla_te3_add_timer(fe, 1000);
-	
+	err=sdla_te3_add_timer(fe, 1000);
+	if (err) {
+		DEBUG_EVENT("%s: Error: Failed to sdla_te3_add_timer() %i\n",fe->name,err);
+	}
 
 	return;
 }
@@ -1405,7 +1495,7 @@ static int sdla_te3_post_init(void* pfe)
 
 
 	/* Initialize and start T1/E1 timer */
-	wan_set_bit(TE_TIMER_KILL,(void*)&fe->te_param.critical);
+	wan_set_bit(TE_TIMER_KILL,(void*)&fe->te3_param.critical);
 	
 	wan_init_timer(
 		&fe->timer, 
@@ -1413,7 +1503,7 @@ static int sdla_te3_post_init(void* pfe)
 	       	(wan_timer_arg_t)fe);
 
 	/* Initialize T1/E1 timer */
-	wan_clear_bit(TE_TIMER_KILL,(void*)&fe->te_param.critical);
+	wan_clear_bit(TE_TIMER_KILL,(void*)&fe->te3_param.critical);
 
 	/* Start T1/E1 timer */
 	if (IS_E3(&fe->fe_cfg)){
@@ -1430,11 +1520,11 @@ static int sdla_te3_pre_release(void* pfe)
 
 
 	/* Kill TE timer poll command */
-	wan_set_bit(TE_TIMER_KILL,(void*)&fe->te_param.critical);
-	if (wan_test_bit(TE_TIMER_RUNNING,(void*)&fe->te_param.critical)){
+	wan_set_bit(TE_TIMER_KILL,(void*)&fe->te3_param.critical);
+	if (wan_test_bit(TE_TIMER_RUNNING,(void*)&fe->te3_param.critical)){
 		wan_del_timer(&fe->timer);
 	}
-	wan_clear_bit(TE_TIMER_RUNNING,(void*)&fe->te_param.critical);
+	wan_clear_bit(TE_TIMER_RUNNING,(void*)&fe->te3_param.critical);
 
 	return 0;
 }
