@@ -936,7 +936,7 @@ static int setup (wan_device_t* wandev, wandev_conf_t* conf)
 
 	case WANCONFIG_AFT_ANALOG:
 		if (card->adptr_type == AFT_ADPTR_A600) {
-                        DEBUG_EVENT("%s: Starting AFT A600 Hardware Init.\n",
+                        DEBUG_EVENT("%s: Starting AFT B600 Hardware Init.\n",
                                                 card->devname);
                         err = wp_aft_a600_init(card,conf);
                 } else {
